@@ -39,8 +39,13 @@
 #ifndef __PIMD_DEFS_H__
 #define __PIMD_DEFS_H__
 
+#ifndef __linux__
+# include <sys/cdefs.h>	/* Defines __BSD_VISIBLE, needed for arc4random() etc. */
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdint.h>
 #include <unistd.h> 
 #include <ctype.h>
 #include <errno.h>
@@ -59,6 +64,7 @@
 #include <sys/time.h>
 #include <net/if.h>
 #include <netinet/in.h>
+#include <arpa/inet.h>
 #include <netinet/in_systm.h>
 #include <netinet/ip.h>
 #include <netinet/igmp.h>
@@ -77,28 +83,17 @@
 #else
 #include <netinet/ip_mroute.h>
 #endif /* __linux__ */
-#if defined(HAVE_STRLCPY)
-#include <string.h>
-#endif
-#if defined(HAVE_STRTONUM)
-#include <stdlib.h>
-#endif
-#if defined(HAVE_PIDFILE)
-#if defined(OpenBSD) || defined(NetBSD)
-#include <util.h>
-#else
-#include <libutil.h>
-#endif
-#endif
+
+/* If using any of the BSD distributions of UNIX the configure script
+ * links with -lutil, but on Linux we link with -lite.  All required
+ * APIs are forward declared in lite.h, so we can use it everywhere. */
+#include "libite/lite.h"
+
 #include <strings.h>
 #ifdef RSRR
 #include <sys/un.h>
 #endif /* RSRR */
 
-typedef u_int   u_int32;
-typedef u_short u_int16;
-typedef u_char  u_int8;
-
 #ifndef BYTE_ORDER
 #if (BSD >= 199103)
 #include <machine/endian.h>
@@ -137,6 +132,7 @@ typedef void (*ihfunc_t) (int, fd_set *);
 #include "pimd.h"
 #include "mrt.h"
 #include "igmpv2.h"
+#include "igmpv3.h"
 #include "vif.h"
 #include "debug.h"
 #include "pathnames.h"
@@ -154,7 +150,7 @@ typedef void (*ihfunc_t) (int, fd_set *);
 #define min(a, b)               ((a) > (b) ? (b) : (a))
 #endif
 
-#define ENABLINGSTR(bool)       (bool) ? "enabling" : "disabling"
+#define ENABLINGSTR(val)        (val) ? "enabling" : "disabling"
 
 /*
  * Various definitions to make it working for different platforms
@@ -166,10 +162,25 @@ typedef void (*ihfunc_t) (int, fd_set *);
 
 /* Versions of Solaris older than 2.6 don't have routing sockets. */
 /* XXX TODO: check FreeBSD version and add all other platforms */
-#if defined(__linux__) || (defined(SunOS) && SunOS >=56) || defined (__FreeBSD__) || defined(__FreeBSD_kernel__) || defined (IRIX) || defined (__bsdi__) || defined(NetBSD) || defined(OpenBSD)
+#if defined(__linux__)    || (defined(SunOS) && SunOS >=56) || \
+    defined (IRIX)        || defined (__bsdi__)             || \
+    defined (__FreeBSD__) || defined(__FreeBSD_kernel__)    || \
+    defined(NetBSD)       || defined(OpenBSD)
 #define HAVE_ROUTING_SOCKETS	1
 #endif
 
+/* Older versions of UNIX don't really give us true raw sockets.
+ * Instead, they expect ip_len and ip_off in host byte order, and also
+ * provide them to us in that format when receiving raw frames.
+ *
+ * This list could probably be made longer, e.g., SunOS and __bsdi__
+ */
+#if defined(__NetBSD__) ||					\
+    (defined(__FreeBSD__) && (__FreeBSD_version < 1100030)) ||	\
+    (defined(__OpenBSD__) && (OpenBSD < 200311))
+#define HAVE_IP_HDRINCL_BSD_ORDER
+#endif
+
 #define TRUE			1
 #define FALSE			0
 
@@ -192,7 +203,8 @@ typedef void (*ihfunc_t) (int, fd_set *);
 #define MINTTL			1  /* min TTL in the packets send locally */
 
 #define MAX_IP_PACKET_LEN       576
-#define MIN_IP_HEADER_LEN       20
+#define MIN_IP_HEADER_LEN       20 /* sizeof(struct ip) */
+#define IP_IGMP_HEADER_LEN      24 /* MIN + Router Alert */
 #define MAX_IP_HEADER_LEN       60
 
 
@@ -202,30 +214,34 @@ typedef void (*ihfunc_t) (int, fd_set *);
  */
 #ifndef INADDR_ALLRTRS_GROUP
 					/* address for multicast mtrace msg */
-#define INADDR_ALLRTRS_GROUP	(u_int32)0xe0000002	/* 224.0.0.2 */
+#define INADDR_ALLRTRS_GROUP	(uint32_t)0xe0000002	/* 224.0.0.2 */
+#endif
+
+#ifndef INADDR_ALLRPTS_GROUP
+#define INADDR_ALLRPTS_GROUP    ((in_addr_t)0xe0000016) /* 224.0.0.22, IGMPv3 */
 #endif
 
 #ifndef INADDR_MAX_LOCAL_GROUP
-#define INADDR_MAX_LOCAL_GROUP	(u_int32)0xe00000ff	/* 224.0.0.255 */
+#define INADDR_MAX_LOCAL_GROUP	(uint32_t)0xe00000ff	/* 224.0.0.255 */
 #endif
 
-#define INADDR_ANY_N            (u_int32)0x00000000     /* INADDR_ANY in
+#define INADDR_ANY_N            (uint32_t)0x00000000     /* INADDR_ANY in
 							 * network order */
-#define CLASSD_PREFIX           (u_int32)0xe0000000     /* 224.0.0.0 */
+#define CLASSD_PREFIX           (uint32_t)0xe0000000     /* 224.0.0.0 */
 #define STAR_STAR_RP_MSKLEN     4                       /* Masklen for
 							 * 224.0.0.0 :
 							 * to encode (*,*,RP)
 							 */
-#define ALL_MCAST_GROUPS_ADDR   (u_int32)0xe0000000     /* 224.0.0.0 */
-#define ALL_MCAST_GROUPS_LENGTH 4
+#define ALL_MCAST_GROUPS_ADDR   (uint32_t)0xe0000000     /* 224.0.0.0 */
+#define ALL_MCAST_GROUPS_LEN    4
 
 /* Used by DVMRP */
 #define DEFAULT_METRIC		1	/* default subnet/tunnel metric     */
 #define DEFAULT_THRESHOLD	1	/* default subnet/tunnel threshold  */
 
 /* Used if no relaible unicast routing information available */
-#define UCAST_DEFAULT_SOURCE_METRIC     1024
-#define UCAST_DEFAULT_SOURCE_PREFERENCE 1024
+#define UCAST_DEFAULT_ROUTE_DISTANCE   101
+#define UCAST_DEFAULT_ROUTE_METRIC     1024
 
 #define TIMER_INTERVAL		5	/* 5 sec virtual timer granularity  */
 
@@ -246,12 +262,16 @@ typedef void (*ihfunc_t) (int, fd_set *);
 #define BIT_TST(X,n)		((X) & 1 << (n))
 #endif /* RSRR */
 
-#ifdef SYSV
-#define setlinebuf(s)		setvbuf((s), (NULL), (_IOLBF), 0)
-#define RANDOM()                lrand48()
+#ifndef __linux__
+#define RANDOM()                arc4random()
 #else
-#define RANDOM()                random()
-#endif /* SYSV */
+#define RANDOM()                (uint32_t)random()
+#endif
+
+/* NetBSD 6.1, for instance, does not have IPOPT_RA defined. */
+#ifndef IPOPT_RA
+#define IPOPT_RA                148
+#endif
 
 /*
  * External declarations for global variables and functions.
@@ -265,6 +285,13 @@ typedef void (*ihfunc_t) (int, fd_set *);
 #define                 SO_RECV_BUF_SIZE_MAX (256*1024)
 #define                 SO_RECV_BUF_SIZE_MIN (48*1024)
 
+
+/*
+ * Global settings, from config.c
+ */
+extern uint16_t         pim_timer_hello_interval;
+extern uint16_t         pim_timer_hello_holdtime;
+
 /* TODO: describe the variables and clean up */
 extern char		*igmp_recv_buf;
 extern char		*igmp_send_buf;
@@ -272,67 +299,82 @@ extern char		*pim_recv_buf;
 extern char		*pim_send_buf;
 extern int		igmp_socket;
 extern int		pim_socket;
-extern u_int32		allhosts_group;
-extern u_int32		allrouters_group;
-extern u_int32		allpimrouters_group;
+extern uint32_t		allhosts_group;
+extern uint32_t		allrouters_group;
+extern uint32_t		allreports_group;
+extern uint32_t		allpimrouters_group;
 extern build_jp_message_t *build_jp_message_pool;
 extern int		build_jp_message_pool_counter;
 
-extern u_long		virtual_time;
-extern char	       *configfilename;
+extern uint32_t		virtual_time;
+extern char	       *config_file;
 extern int              haveterminal;
 extern char            *__progname;
 
 extern struct cand_rp_adv_message_ {
-    u_int8    *buffer;
-    u_int8    *insert_data_ptr;
-    u_int8    *prefix_cnt_ptr;
-    u_int16   message_size;
+    uint8_t  *buffer;
+    uint8_t  *insert_data_ptr;
+    uint8_t  *prefix_cnt_ptr;
+    uint16_t  message_size;
 } cand_rp_adv_message;
 
 extern int disable_all_by_default;
+extern int mrt_table_id;
 
 /*
  * Used to contol the switching to the shortest path:
- *  `reg_rate`  used by the RP
- *  `data_rate` used by the last hop router
  */
-extern u_int32		pim_reg_rate_bytes;
-extern u_int32		pim_reg_rate_check_interval;
-extern u_int32		pim_data_rate_bytes;
-extern u_int32		pim_data_rate_check_interval;
+typedef enum {
+    SPT_RATE,
+    SPT_PACKETS,
+    SPT_INF
+} spt_mode_t;
+
+typedef struct {
+    uint8_t   mode;
+    uint32_t  bytes;
+    uint32_t  packets;
+    uint32_t  interval;
+} spt_threshold_t;
+extern spt_threshold_t  spt_threshold;
 
 extern cand_rp_t        *cand_rp_list;
 extern grp_mask_t       *grp_mask_list;
 extern cand_rp_t        *segmented_cand_rp_list;
 extern grp_mask_t       *segmented_grp_mask_list;
 
-extern u_int16          curr_bsr_fragment_tag;
-extern u_int8           curr_bsr_priority;
-extern u_int32          curr_bsr_address;
-extern u_int32          curr_bsr_hash_mask;
-extern u_int8		cand_bsr_flag;		   /* candidate BSR flag */
-extern u_int8           my_bsr_priority;
-extern u_int32          my_bsr_address;
-extern u_int32          my_bsr_hash_mask;
-extern u_int8           cand_rp_flag;              /* Candidate RP flag */
-extern u_int32          my_cand_rp_address;
-extern u_int8           my_cand_rp_priority;
-extern u_int16          my_cand_rp_holdtime;
-extern u_int16          my_cand_rp_adv_period;     /* The locally configured
+extern uint16_t          curr_bsr_fragment_tag;
+extern uint8_t           curr_bsr_priority;
+extern uint32_t          curr_bsr_address;
+extern uint32_t          curr_bsr_hash_mask;
+extern uint8_t		 cand_bsr_flag;		   /* candidate BSR flag */
+extern uint8_t           my_bsr_priority;
+extern uint32_t          my_bsr_address;
+extern uint32_t          my_bsr_hash_mask;
+extern uint8_t           cand_rp_flag;              /* Candidate RP flag */
+extern uint32_t          my_cand_rp_address;
+extern uint8_t           my_cand_rp_priority;
+extern uint16_t          my_cand_rp_holdtime;
+extern uint16_t          my_cand_rp_adv_period;     /* The locally configured
 						    * Cand-RP adv. period.
 						    */
-extern u_int16          pim_bootstrap_timer;
-extern u_int32          rp_my_ipv4_hashmask;
-extern u_int16          pim_cand_rp_adv_timer;
+extern uint16_t          pim_bootstrap_timer;
+extern uint32_t          rp_my_ipv4_hashmask;
+extern uint16_t          pim_cand_rp_adv_timer;
 
-extern u_int32		default_source_metric;
-extern u_int32		default_source_preference;
+/* route.c */
+extern uint32_t		default_route_metric;
+extern uint32_t		default_route_distance;
 
+/* igmp_proto.c */
+extern uint32_t		igmp_query_interval;
+extern uint32_t		igmp_querier_timeout;
+
+/* mrt.c */
 extern srcentry_t 	*srclist;
 extern grpentry_t 	*grplist;
-extern rpentry_t        *rplist;
 
+/* vif.c */
 extern struct uvif	uvifs[MAXVIFS];
 extern vifi_t		numvifs;
 extern int              total_interfaces;
@@ -364,6 +406,11 @@ extern int		errno;
 #endif
 #define IGMP_V2_LEAVE_GROUP		IGMP_HOST_LEAVE_MESSAGE
 #endif
+#if defined(__FreeBSD__)		/* From FreeBSD 8.x */
+#define IGMP_V3_MEMBERSHIP_REPORT       IGMP_v3_HOST_MEMBERSHIP_REPORT
+#else
+#define IGMP_V3_MEMBERSHIP_REPORT	0x22	/* Ver. 3 membership report */
+#endif
 
 #if defined(NetBSD) || defined(OpenBSD) || defined(__FreeBSD__)
 #define IGMP_MTRACE_RESP		IGMP_MTRACE_REPLY
@@ -390,24 +437,10 @@ extern int		errno;
 #define NOT_TIMEOUT(timer)		\
 	((timer) -= (MIN(timer, TIMER_INTERVAL)))
 
-#if 0
-#define IF_TIMEOUT(value)     \
-  if (!(((value) >= TIMER_INTERVAL) && ((value) -= TIMER_INTERVAL)))
-
-#define IF_NOT_TIMEOUT(value) \
-  if (((value) >= TIMER_INTERVAL) && ((value) -= TIMER_INTERVAL))
-
-#define TIMEOUT(value)        \
-     (!(((value) >= TIMER_INTERVAL) && ((value) -= TIMER_INTERVAL)))
-
-#define NOT_TIMEOUT(value)    \
-     (((value) >= TIMER_INTERVAL) && ((value) -= TIMER_INTERVAL))
-#endif /* 0 */
-
 #define ELSE else           /* To make emacs cc-mode happy */      
 
 #define MASK_TO_VAL(x, i) {		   \
-	u_int32 _x = ntohl(x);		   \
+	uint32_t _x = ntohl(x);		   \
 	(i) = 1;			   \
 	while ((_x) <<= 1)		   \
 	    (i)++;			   \
@@ -435,10 +468,11 @@ extern void	config_vifs_from_kernel	(void);
 extern void	config_vifs_from_file	(void);
 
 /* debug.c */
-extern char	*packet_kind		(u_int proto, u_int type, u_int code);
-extern int	debug_kind		(u_int proto, u_int type, u_int code);
+extern char	*packet_kind		(int proto, int type, int code);
+extern int	debug_kind		(int proto, int type, int code);
 extern void	logit			(int, int, const char *, ...);
-extern int	log_level		(u_int proto, u_int type, u_int code);
+extern void	dump_frame		(char *desc, void *dump, size_t len);
+extern int	log_level		(int proto, int type, int code);
 extern void	dump			(int i);
 extern void	fdump			(int i);
 extern void	cdump			(int i);
@@ -447,142 +481,142 @@ extern void	dump_pim_mrt		(FILE *fp);
 extern int	dump_rp_set		(FILE *fp);
 
 /* dvmrp_proto.c */
-extern void	dvmrp_accept_probe	(u_int32 src, u_int32 dst, u_char *p, int datalen, u_int32 level);
-extern void	dvmrp_accept_report	(u_int32 src, u_int32 dst, u_char *p, int datalen, u_int32 level);
-extern void	dvmrp_accept_info_request (u_int32 src, u_int32 dst, u_char *p, int datalen);
-extern void	dvmrp_accept_info_reply	(u_int32 src, u_int32 dst, u_char *p, int datalen);
-extern void	dvmrp_accept_neighbors	(u_int32 src, u_int32 dst, u_char *p, int datalen, u_int32 level);
-extern void	dvmrp_accept_neighbors2	(u_int32 src, u_int32 dst, u_char *p, int datalen, u_int32 level);
-extern void	dvmrp_accept_prune	(u_int32 src, u_int32 dst, u_char *p, int datalen);
-extern void	dvmrp_accept_graft	(u_int32 src, u_int32 dst, u_char *p, int datalen);
-extern void	dvmrp_accept_g_ack	(u_int32 src, u_int32 dst, u_char *p, int datalen);
+extern void	dvmrp_accept_probe	(uint32_t src, uint32_t dst, uint8_t *p, int datalen, uint32_t level);
+extern void	dvmrp_accept_report	(uint32_t src, uint32_t dst, uint8_t *p, int datalen, uint32_t level);
+extern void	dvmrp_accept_info_request (uint32_t src, uint32_t dst, uint8_t *p, int datalen);
+extern void	dvmrp_accept_info_reply	(uint32_t src, uint32_t dst, uint8_t *p, int datalen);
+extern void	dvmrp_accept_neighbors	(uint32_t src, uint32_t dst, uint8_t *p, int datalen, uint32_t level);
+extern void	dvmrp_accept_neighbors2	(uint32_t src, uint32_t dst, uint8_t *p, int datalen, uint32_t level);
+extern void	dvmrp_accept_prune	(uint32_t src, uint32_t dst, uint8_t *p, int datalen);
+extern void	dvmrp_accept_graft	(uint32_t src, uint32_t dst, uint8_t *p, int datalen);
+extern void	dvmrp_accept_g_ack	(uint32_t src, uint32_t dst, uint8_t *p, int datalen);
 
 /* igmp.c */
 extern void	init_igmp		(void);
-extern void	send_igmp		(char *buf, u_int32 src, u_int32 dst, int type, int code, u_int32 group, int datalen);
+extern void	send_igmp		(char *buf, uint32_t src, uint32_t dst, int type, int code, uint32_t group, int datalen);
 
 /* igmp_proto.c */
 extern void	query_groups		(struct uvif *v);
-extern void	accept_membership_query	(u_int32 src, u_int32 dst, u_int32 group, int tmo);
-extern void	accept_group_report	(u_int32 src, u_int32 dst, u_int32 group, int r_type);
-extern void	accept_leave_message	(u_int32 src, u_int32 dst, u_int32 group);
+extern void	accept_membership_query	(uint32_t src, uint32_t dst, uint32_t group, int tmo, int igmp_version);
+extern void	accept_group_report	(uint32_t src, uint32_t dst, uint32_t group, int r_type);
+extern void	accept_leave_message	(uint32_t src, uint32_t dst, uint32_t group);
+extern void     accept_membership_report(uint32_t src, uint32_t dst, struct igmpv3_report *report, ssize_t reportlen);
 
 /* inet.c */
-extern int	inet_cksum		(u_int16 *addr, u_int len);
-extern int	inet_valid_host		(u_int32 naddr);
-extern int	inet_valid_mask		(u_int32 mask);
-extern int	inet_valid_subnet	(u_int32 nsubnet, u_int32 nmask);
-extern char	*inet_fmt		(u_int32 addr, char *s, size_t len);
-extern char	*netname		(u_int32 addr, u_int32 mask);
-extern u_int32	inet_parse		(char *s, int n);
+extern int	inet_cksum		(uint16_t *addr, u_int len);
+extern int	inet_valid_host		(uint32_t naddr);
+extern int	inet_valid_mask		(uint32_t mask);
+extern int	inet_valid_subnet	(uint32_t nsubnet, uint32_t nmask);
+extern char	*inet_fmt		(uint32_t addr, char *s, size_t len);
+extern char	*netname		(uint32_t addr, uint32_t mask);
+extern uint32_t	inet_parse		(char *s, int n);
 
 /* kern.c */
 extern void	k_set_sndbuf		(int socket, int bufsize, int minsize);
 extern void	k_set_rcvbuf		(int socket, int bufsize, int minsize);
-extern void	k_hdr_include		(int socket, int bool);
+extern void	k_hdr_include		(int socket, int val);
 extern void	k_set_ttl		(int socket, int t);
 extern void	k_set_loop		(int socket, int l);
-extern void	k_set_if		(int socket, u_int32 ifa);
-extern void	k_join			(int socket, u_int32 grp, struct uvif *v);
-extern void	k_leave			(int socket, u_int32 grp, struct uvif *v);
+extern void	k_set_if		(int socket, uint32_t ifa);
+extern void	k_set_router_alert	(int socket);
+extern void	k_join			(int socket, uint32_t grp, struct uvif *v);
+extern void	k_leave			(int socket, uint32_t grp, struct uvif *v);
 extern void	k_init_pim		(int socket);
 extern void	k_stop_pim		(int socket);
-extern int	k_del_mfc		(int socket, u_int32 source, u_int32 group);
-extern int	k_chg_mfc		(int socket, u_int32 source, u_int32 group, vifi_t iif, vifbitmap_t oifs,
-                                         u_int32 rp_addr);
+extern int	k_del_mfc		(int socket, uint32_t source, uint32_t group);
+extern int	k_chg_mfc		(int socket, uint32_t source, uint32_t group, vifi_t iif, vifbitmap_t oifs,
+                                         uint32_t rp_addr);
 extern void	k_add_vif		(int socket, vifi_t vifi, struct uvif *v);
 extern void	k_del_vif		(int socket, vifi_t vifi, struct uvif *v);
 extern int	k_get_vif_count		(vifi_t vifi, struct vif_count *retval);
-extern int	k_get_sg_cnt		(int socket, u_int32 source, u_int32 group, struct sg_count *retval);
+extern int	k_get_sg_cnt		(int socket, uint32_t source, uint32_t group, struct sg_count *retval);
 
 /* main.c */
 extern int	register_input_handler	(int fd, ihfunc_t func);
 
 /* mrt.c */
 extern void	init_pim_mrt		(void);
-extern mrtentry_t *find_route		(u_int32 source, u_int32 group, u_int16 flags, char create);
-extern grpentry_t *find_group		(u_int32 group);
-extern srcentry_t *find_source		(u_int32 source);
+extern mrtentry_t *find_route		(uint32_t source, uint32_t group, uint16_t flags, char create);
+extern grpentry_t *find_group		(uint32_t group);
+extern srcentry_t *find_source		(uint32_t source);
 extern void	delete_mrtentry		(mrtentry_t *mrtentry_ptr);
 extern void	delete_srcentry		(srcentry_t *srcentry_ptr);
 extern void	delete_grpentry		(grpentry_t *grpentry_ptr);
 extern void	delete_mrtentry_all_kernel_cache (mrtentry_t *mrtentry_ptr);
 extern void	delete_single_kernel_cache (mrtentry_t *mrtentry_ptr, kernel_cache_t *kernel_cache_ptr);
-extern void	delete_single_kernel_cache_addr (mrtentry_t *mrtentry_ptr, u_int32 source, u_int32 group);
-extern void	add_kernel_cache	(mrtentry_t *mrtentry_ptr, u_int32 source, u_int32 group, u_int16 flags);
+extern void	delete_single_kernel_cache_addr (mrtentry_t *mrtentry_ptr, uint32_t source, uint32_t group);
+extern void	add_kernel_cache	(mrtentry_t *mrtentry_ptr, uint32_t source, uint32_t group, uint16_t flags);
 /* pim.c */
 extern void	init_pim		(void);
-extern void	send_pim		(char *buf, u_int32 src, u_int32 dst, int type, int datalen);
-extern void	send_pim_unicast	(char *buf, u_int32 src, u_int32 dst, int type, int datalen);
+extern void	send_pim		(char *buf, uint32_t src, uint32_t dst, int type, size_t len);
+extern void	send_pim_unicast	(char *buf, int mtu, uint32_t src, uint32_t dst, int type, size_t len);
 
 /* pim_proto.c */
-extern int	receive_pim_hello	(u_int32 src, u_int32 dst, char *pim_message, size_t datalen);
-extern int	send_pim_hello		(struct uvif *v, u_int16 holdtime);
+extern int	receive_pim_hello	(uint32_t src, uint32_t dst, char *msg, size_t len);
+extern int	send_pim_hello		(struct uvif *v, uint16_t holdtime);
 extern void	delete_pim_nbr		(pim_nbr_entry_t *nbr_delete);
-extern int	receive_pim_register	(u_int32 src, u_int32 dst, char *pim_message, size_t datalen);
+extern int	receive_pim_register	(uint32_t src, uint32_t dst, char *msg, size_t len);
 extern int	send_pim_null_register	(mrtentry_t *r);
-extern int	receive_pim_register_stop (u_int32 src, u_int32 dst, char *pim_message, size_t datalen);
+extern int	receive_pim_register_stop (uint32_t src, uint32_t dst, char *msg, size_t len);
 extern int	send_pim_register	(char *pkt);
-extern int	receive_pim_join_prune	(u_int32 src, u_int32 dst, char *pim_message, int datalen);
+extern int	receive_pim_join_prune	(uint32_t src, uint32_t dst, char *msg, size_t len);
 extern int	join_or_prune		(mrtentry_t *mrtentry_ptr, pim_nbr_entry_t *upstream_router);
-extern int	receive_pim_assert	(u_int32 src, u_int32 dst, char *pim_message, int datalen);
-extern int	send_pim_assert		(u_int32 source, u_int32 group, vifi_t vifi, mrtentry_t *mrtentry_ptr);
-extern int	send_periodic_pim_join_prune (vifi_t vifi, pim_nbr_entry_t *pim_nbr, u_int16 holdtime);
-extern int	add_jp_entry		(pim_nbr_entry_t *pim_nbr, u_int16 holdtime, u_int32 group, u_int8 grp_msklen,
-                                         u_int32 source, u_int8 src_msklen,  u_int16 addr_flags, u_int8 join_prune);
+extern int	receive_pim_assert	(uint32_t src, uint32_t dst, char *msg, size_t len);
+extern int	send_pim_assert		(uint32_t source, uint32_t group, vifi_t vifi, mrtentry_t *mrtentry_ptr);
+extern int	send_periodic_pim_join_prune (vifi_t vifi, pim_nbr_entry_t *pim_nbr, uint16_t holdtime);
+extern int	add_jp_entry		(pim_nbr_entry_t *pim_nbr, uint16_t holdtime, uint32_t group, uint8_t grp_msklen,
+                                         uint32_t source, uint8_t src_msklen,  uint16_t addr_flags, uint8_t join_prune);
 extern void	pack_and_send_jp_message (pim_nbr_entry_t *pim_nbr);
-extern int	receive_pim_cand_rp_adv	(u_int32 src, u_int32 dst, char *pim_message, int datalen);
-extern int	receive_pim_bootstrap	(u_int32 src, u_int32 dst, char *pim_message, int datalen);
+extern int	receive_pim_cand_rp_adv	(uint32_t src, uint32_t dst, char *msg, size_t len);
+extern int	receive_pim_bootstrap	(uint32_t src, uint32_t dst, char *msg, size_t len);
 extern int	send_pim_cand_rp_adv	(void);
 extern void	send_pim_bootstrap	(void);
 
 /* route.c */
 extern int	set_incoming		(srcentry_t *srcentry_ptr, int srctype);
-extern vifi_t	get_iif			(u_int32 source);
-extern pim_nbr_entry_t *find_pim_nbr	(u_int32 source);
-extern int	add_sg_oif		(mrtentry_t *mrtentry_ptr, vifi_t vifi, u_int16 holdtime, int update_holdtime);
-extern void	add_leaf		(vifi_t vifi, u_int32 source, u_int32 group);
-extern void	delete_leaf		(vifi_t vifi, u_int32 source, u_int32 group);
+extern vifi_t	get_iif			(uint32_t source);
+extern pim_nbr_entry_t *find_pim_nbr	(uint32_t source);
+extern int	add_sg_oif		(mrtentry_t *mrtentry_ptr, vifi_t vifi, uint16_t holdtime, int update_holdtime);
+extern void	add_leaf		(vifi_t vifi, uint32_t source, uint32_t group);
+extern void	delete_leaf		(vifi_t vifi, uint32_t source, uint32_t group);
 extern int	change_interfaces	(mrtentry_t *mrtentry_ptr,  vifi_t new_iif,
                                          vifbitmap_t new_joined_oifs_, vifbitmap_t new_pruned_oifs,
-                                         vifbitmap_t new_leaves_, vifbitmap_t new_asserted_oifs, u_int16 flags);
+                                         vifbitmap_t new_leaves_, vifbitmap_t new_asserted_oifs, uint16_t flags);
 extern void	calc_oifs		(mrtentry_t *mrtentry_ptr, vifbitmap_t *oifs_ptr);
 extern void	process_kernel_call	(void);
 extern int	delete_vif_from_mrt	(vifi_t vifi);
-extern mrtentry_t *switch_shortest_path	(u_int32 source, u_int32 group);
+extern mrtentry_t *switch_shortest_path	(uint32_t source, uint32_t group);
 
-/* routesock.c */
-extern int	k_req_incoming		(u_int32 source, struct rpfctl *rpfp);
-#ifdef HAVE_ROUTING_SOCKETS
+/* routesock.c and netlink.c */
+extern int	k_req_incoming		(uint32_t source, struct rpfctl *rpfp);
 extern int	init_routesock		(void);
 extern int	routing_socket;
-#endif /* HAVE_ROUTING_SOCKETS */
 
 /* rp.c */
 extern void	init_rp_and_bsr		(void);
-extern u_int16	bootstrap_initial_delay (void);
+extern uint16_t	bootstrap_initial_delay (void);
 extern rp_grp_entry_t *add_rp_grp_entry (cand_rp_t  **used_cand_rp_list,
                                          grp_mask_t **used_grp_mask_list,
-                                         u_int32 rp_addr,
-                                         u_int8  rp_priority,
-                                         u_int16 rp_holdtime,
-                                         u_int32 group_addr,
-                                         u_int32 group_mask,
-                                         u_int32 bsr_hash_mask,
-                                         u_int16 fragment_tag);
+                                         uint32_t rp_addr,
+                                         uint8_t  rp_priority,
+                                         uint16_t rp_holdtime,
+                                         uint32_t group_addr,
+                                         uint32_t group_mask,
+                                         uint32_t bsr_hash_mask,
+                                         uint16_t fragment_tag);
 extern void	delete_rp_grp_entry	(cand_rp_t  **used_cand_rp_list, grp_mask_t **used_grp_mask_list,
                                          rp_grp_entry_t *rp_grp_entry_delete);
 extern void	delete_grp_mask		(cand_rp_t  **used_cand_rp_list, grp_mask_t **used_grp_mask_list,
-                                         u_int32 group_addr, u_int32 group_mask);
+                                         uint32_t group_addr, uint32_t group_mask);
 extern void	delete_rp		(cand_rp_t  **used_cand_rp_list, grp_mask_t **used_grp_mask_list,
-                                         u_int32 rp_addr);
+                                         uint32_t rp_addr);
 extern void	delete_rp_list		(cand_rp_t  **used_cand_rp_list, grp_mask_t **used_grp_mask_list);
-extern rpentry_t *rp_match		(u_int32 group);
-extern rp_grp_entry_t *rp_grp_match	(u_int32 group);
-extern rpentry_t *rp_find		(u_int32 rp_address);
+extern rpentry_t *rp_match		(uint32_t group);
+extern rp_grp_entry_t *rp_grp_match	(uint32_t group);
+extern rpentry_t *rp_find		(uint32_t rp_address);
 extern int	remap_grpentry		(grpentry_t *grpentry_ptr);
 extern int	create_pim_bootstrap_message (char *send_buff);
-extern int	check_mrtentry_rp	(mrtentry_t *mrtentry_ptr, u_int32 rp_addr);
+extern int	check_mrtentry_rp	(mrtentry_t *mrtentry_ptr, uint32_t rp_addr);
 
 #ifdef RSRR
 #ifdef PIM
@@ -608,36 +642,29 @@ extern int	unicast_routing_changes	(srcentry_t *src_ent);
 extern int	clean_srclist		(void);
 
 /* trace.c */
-/* u_int is promoted u_char */
-extern void	accept_mtrace		(u_int32 src, u_int32 dst, u_int32 group, char *data, u_int no, int datalen);
-extern void	accept_neighbor_request	(u_int32 src, u_int32 dst);
-extern void	accept_neighbor_request2 (u_int32 src, u_int32 dst);
+/* u_int is promoted uint8_t */
+extern void	accept_mtrace		(uint32_t src, uint32_t dst, uint32_t group, char *data, u_int no, int datalen);
+extern void	accept_neighbor_request	(uint32_t src, uint32_t dst);
+extern void	accept_neighbor_request2 (uint32_t src, uint32_t dst);
 
 /* vif.c */
 extern void	init_vifs		(void);
 extern void	zero_vif		(struct uvif *, int);
 extern void	stop_all_vifs		(void);
 extern void	check_vif_state		(void);
-extern vifi_t	local_address		(u_int32 src);
-extern vifi_t	find_vif_direct		(u_int32 src);
-extern vifi_t	find_vif_direct_local	(u_int32 src);
-extern u_int32	max_local_address	(void);
+extern vifi_t	local_address		(uint32_t src);
+extern vifi_t	find_vif_direct		(uint32_t src);
+extern vifi_t	find_vif_direct_local	(uint32_t src);
+extern uint32_t	max_local_address	(void);
 
 struct rp_hold {
 	struct rp_hold *next;
-	u_int32	address;
-	u_int32	group;
-	u_int32	mask;
-	u_int8	priority;
+	uint32_t	address;
+	uint32_t	group;
+	uint32_t	mask;
+	uint8_t	priority;
 };
 
-#ifndef HAVE_STRLCPY
-size_t strlcpy(char *dst, const char *src, size_t siz);
-#endif
-#ifndef HAVE_PIDFILE
-int pidfile(const char *basename);
-#endif
-
 #endif /* __PIMD_DEFS_H__ */
 
 /**
